#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#define SOFT_FAILURE

/* global variables */
static int (*o_socket) (int domain, int type, int protocol);

static void dlinit(void)
{
    char *error;

    o_socket = dlsym(RTLD_NEXT, "socket");

    if ((error = dlerror()) != NULL)
	fprintf(stderr, "dlsym: %s", error), exit(255);
}

int socket(int domain, int type, int protocol) {
    static int loaded = 0;
    int fd;

    if (!loaded) {
	dlinit();
	loaded = 1;
    }

    fd = o_socket(domain, type, protocol);

    if (fd != -1 && domain == AF_INET6) {
	int opt = 0;

	if (setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &opt, sizeof(opt)) < 0) {
#ifdef SOFT_FAILURE
	    close(fd);
	    errno = EAFNOSUPPORT;
	    fd = -1;
#else
	    perror("setsockopt(IPV6_V6ONLY)");
	    exit(255);
#endif
	}
    }

    return fd;
}

